' Enigma Revisited Demo
' by Krischan 2012

SuperStrict

Import "amiga.o"

Framework BRL.GLMax2D

Import BRL.DirectSoundAudio  ' Windows sound output
Import BRL.OGGloader         ' OGG support
Import BRL.Timer             ' Timer functions
Import BRL.FreeTypeFont 
Import SIdesign.MiniB3D      ' 3D functions

Incbin "gfx/phenomena.png"
Incbin "gfx/mask.png"
Incbin "gfx/mandelbrot.jpg"
Incbin "gfx/haze.png"
Incbin "gfx/small_pixel.ttf"

Incbin "music/hyperbased.ogg"
Incbin "music/hyperbased_sample.ogg"

AppTitle = "Enigma Revisited"

Extern "win32"
	Function ExtractIconA:Int(hWnd:Int, File$z, Index:Int)
	Function GetActiveWindow%()
	Function SendMessage:Int(hWnd:Int, MSG:Int, wParam:Int, lParam:Int) = "SendMessageA@16"
End Extern

Graphics3D 1024,600,32,2

Type TBall

	Global List:TList = CreateList()

	Method New()

		ListAddLast(List, Self)

	End Method

	Field e:TMesh
	Field i:Int
	Field x:Float
	Field r:Int,g:Int,b:Int

End Type

Local timer:TTimer=CreateTimer(60)

Local pivot:TPivot=CreatePivot()
PositionEntity pivot,-3,2,0

Local cam:TCamera=CreateCamera(pivot)
PositionEntity cam,-10,0,-5
CameraRange cam,0.01,32*256
CameraFogMode cam,True
CameraFogColor cam,255,0,0
CameraFogRange cam,12,32
PointEntity cam,pivot

Local light:TLight=CreateLight(2)
PositionEntity light,-16,16,-16
AmbientLight 8,8,8

Local plane:TSprite=CreateSprite()
SpriteViewMode plane,2
RotateEntity plane,90,0,0
PositionEntity plane,0,0,0
ScaleSprite plane,24,24
EntityFX plane,1+2+32

' fade quad
Local fade:TSprite=CreateSprite(cam)
SpriteViewMode fade,2
EntityFX fade,1+2+8+32
EntityColor fade,0,0,0
PositionEntity fade,0,0,0.1
EntityOrder fade,-100

Local planetex:TTexture=LoadTexture("incbin::gfx/mandelbrot.jpg")
Local masktex:TTexture=LoadTexture("incbin::gfx/mask.png",2)
EntityTexture plane,masktex,0,1
EntityTexture plane,planetex,0,2

For Local i:Int=0 To 31

	Local balls:TBall = New TBall

	balls.e=CreateSphere(32)
	balls.i=i
	balls.x=i

	EntityFX balls.e,2
	EntityShininess balls.e,1

	UpdateMesh(balls.e,0,255,0,1)

Next

Local haze:TMesh=InitHaze(64,48,32,1+2+32,255,0,0,0,1)
PositionEntity haze,0,-2,0
EntityOrder haze,1
Local hazetex:TTexture=LoadTexture("incbin::gfx/haze.png",2)
EntityTexture haze,hazetex,0,1

Local sample:TSound=LoadSound("incbin::music/hyperbased_sample.ogg")
Local music:TSound=LoadSound("incbin::music/hyperbased.ogg",SOUND_LOOP)

Local logopivot:TPivot=CreatePivot(cam)
PositionEntity logopivot,0,GraphicsHeight()-128,GraphicsWidth()

Local logo:TTexture=LoadTexture("incbin::gfx/phenomena.png",2)

Local logodummy:TSprite[64+1]

For Local i:Int=1 To 64

	logodummy[i]=CreateSprite(logopivot)
	ScaleSprite logodummy[i],GraphicsWidth(),GraphicsWidth()*128/1024.0
	EntityTexture logodummy[i],logo
	SpriteViewMode logodummy[i],2
	TranslateEntity logodummy[i],0,0,(i*0.25)^2
	EntityFX logodummy[i],1+2+8+16+32
	UpdateMesh(logodummy[i],255,255,255,(1-(i*1.0/64)))

	If i>1 Then HideEntity logodummy[i]

Next

Local d:Int=180.0
Local e:Int=0.0
Local f:Float=1.0
Local l:Int=1
Local channel:TChannel

Local red:Int,green:Int,blue:Int

Local start:Int=MilliSecs()

SetIcon(AppFile, GetActiveWindow())

Local intro:TChannel=PlaySound(sample)

Local font:TImagefont = LoadImageFont( "incbin::gfx/small_pixel.ttf", 8)

SetImageFont(font)

RenderWorld
Flip

CameraClsColor cam,0,0,128

While Not AppTerminate()

	If KeyHit(KEY_Escape) Then End

	' fade screen
	If f>0 Then
		f=f-0.005
		UpdateMesh(fade,0,0,0,f)
	EndIf

	If ChannelPlaying(intro)=0 Or KeyHit(KEY_SPACE) Then

		StopChannel intro

		If Not channel Then channel=PlaySound(music)

		d=d+1
		If d>360 Then d=0

		l=l+1
		If l>64 Then l=64

		If l=64 Then e=e+1

		Local s:Float=Sin(d)
		Local c:Float=Cos(d)

		ShowEntity logodummy[l]
		
		' movement math
		TurnEntity pivot,0,(s+c)+(EntityY(cam,1)/Pi),0
		If l=64 Then RotateEntity logopivot,Sin(e)*4,Sin(e)*4,(Sin(e)+Sin(c))*4
		TranslateEntity cam,0,c/Pi/10.0,0
		PointEntity cam,pivot

		TurnEntity cam,5,0,0

		red=Abs(Sin(d/2.0))*255
		green=Abs(Cos(d/2.0))*255
		blue=Abs(Sin(d/2.0)+Cos(d/2.0))*255

		CameraFogColor cam,red,green,blue

	EndIf

	RenderWorld

	' ball movement
	For Local b:TBall = EachIn TBall.list

		b.x=b.x-0.05
		If b.x<0 Then b.x=31

		PositionEntity b.e,b.x-15,Sin((b.x-15)*34)*2,0

	Next
	
	BeginMax2D()

		DrawTextCenter("Original Music by Firefox + Tip      Remix by Moonove      Coding by Krischan",GraphicsWidth()/2,GraphicsHeight()-TextHeight("X")-1)

	EndMax2D()

	WaitTimer timer
		
	Flip False
	
Wend

End

Function SetIcon(iconname$, TheWindow%)	
	?Win32
	Local icon:Int = ExtractIconA(TheWindow, iconname, 0)
	Local WM_SETICON:Int = $80
	Local ICON_BIG:Int = 1
	sendmessage(TheWindow, WM_SETICON, ICON_BIG, icon)
	?
End Function

Function DrawTextCenter(text:String,x:Int,y:Int)

	Local w:Float=TextWidth(text)

	SetBlend ALPHABLEND

	SetColor 0,0,0
	DrawText(text,x-(w/2)+1,y+1)
	SetColor 255,255,255
	DrawText(text,x-(w/2),y)

End Function 

Function UpdateMesh(mesh:TMesh,r:Int=255,g:Int=255,b:Int=255,a:Float=-1)
	
	Local v:Int
	Local s:Int
	Local surf:TSurface
	Local alpha:Float

	For s=1 To CountSurfaces(mesh)

		surf=GetSurface(mesh,s)
	
		For v=0 To CountVertices(surf)-1

			If a<0 Then
				alpha=VertexAlpha(surf,v)
			Else
				alpha=a
			EndIf
		
			VertexColor surf,v,r,g,b,alpha
		
		Next

	Next
	
End Function

Function InitHaze:TMesh(radius:Float,h:Float,segments:Int,fx:Int=0,r:Int=255,g:Int=255,b:Int=255,a1:Float=0.0,a2:Float=1.0)
	
	Local ang1:Float,ang2:Float,inc:Float,x1:Float,z1:Float,x2:Float,z2:Float
	Local uu1:Float,uu2:Float,uv1:Float,uv2:Float,uv3:Float
	Local v0:Int,v1:Int,v2:Int,v3:Int,v4:Int,v5:Int
	
	Local mesh:TMesh=CreateMesh()
	Local surf:TSurface=CreateSurface(mesh)
	
	inc=360.0/segments
	
	' complete circle
	While ang1<360
		

		' Angle
		ang2=(ang1+inc) Mod 360
		
		' Vertex coordinates
		x1=radius*Cos(ang1)
		z1=radius*Sin(ang1)
		x2=radius*Cos(ang2)
		z2=radius*Sin(ang2)
		
		' UV-coordinates
		uu1=1.0-(ang1/360.0)
		uu2=1.0-(ang1+inc)/360.0
		uv1=1.00
		uv2=0.50
		uv3=0.00
		
		' add vertices
		v0=AddVertex(surf,x1,-h,z1,uu1,uv1)
		v1=AddVertex(surf,x2,-h,z2,uu2,uv1)
		v2=AddVertex(surf,x1, 0,z1,uu1,uv2)
		v3=AddVertex(surf,x2, 0,z2,uu2,uv2)
		v4=AddVertex(surf,x1, h,z1,uu1,uv3)
		v5=AddVertex(surf,x2, h,z2,uu2,uv3)
		
		' add vertex color
		VertexColor surf,v0,r,g,b,a1
		VertexColor surf,v1,r,g,b,a1
		VertexColor surf,v2,r,g,b,a2
		VertexColor surf,v3,r,g,b,a2
		VertexColor surf,v4,r,g,b,a1
		VertexColor surf,v5,r,g,b,a1

		' add triangles
		AddTriangle surf,v0,v1,v3
		AddTriangle surf,v3,v2,v0
		AddTriangle surf,v2,v5,v4
		AddTriangle surf,v5,v2,v3
		
		' increase angle
		ang1=ang1+inc
		
	Wend
	
	EntityFX mesh,fx
	
	Return mesh
	
End Function